;---------------------------------------------------------------------------
;
; 2600 RPG
;
;---------------------------------------------------------------------------

	processor 6502	
	include vcs.h	


;---------------------------------------------------------------------------
; Todo
;---------------------------------------------------------------------------
;
;
; - Create full test level
; - Add support for random rocks
; - Add remaining 1 byte support
;
; - Expand max world size to 4 pages
; - Add support for multiple worlds
;
; - Fix $C7 accidental write?
;
; LATER:
;
; - Add warp capability
; - Add map opening/closing animation
; - Optimize program for ROM space
; - Organize and optimize RAM
;
;

;===========================================================================
; Constants
;===========================================================================

TOPBLANKTIME equ 21		; Time x 64 cycles of blank area above map

PLAYERCOLOR equ $08		; Color of player graphics/map

VBLANKOVERSCANTIME equ 17

STARTPOSX equ $20
STARTPOSY equ $20

MANOFFSET equ 22

MAPSIZEX equ $70
MAPSIZEY equ $70

;---------------------------------
; Mode definitions / counter settings
;---------------------------------
M_MAP		equ 8	; display map normally

M_LOAD	equ 9	; load map

M_SCRL	equ 0 ; scroll left
M_SCRR	equ 1	; scroll right
M_SCRU	equ 2	; scroll up
M_SCRD	equ 3	; scroll down

M_SCRLS	equ 4 ; scroll left special
M_SCRRS	equ 5 ; (unused)
M_SCRUS	equ 6	; scroll up special
M_SCRDS	equ 7 ; (unused)

SCROLL_MASK equ %11111000 ; mask to check for any scroll mode

; Mode frame counts
C_LOAD	equ 11
C_SCR		equ 2 ; scroll count



;===========================================================================
; RAM Variables 
;===========================================================================

;---------------------------------
; levelData block (50 bytes)
;---------------------------------
levelDataA	equ $80

;---------------------------------
; grfxBuffer and overlapping temps
; (20 bytes)							;
;---------------------------------
temp3			equ $B2					;
startX		equ $B4					;
width			equ $B6					;
startY		equ $B8					;
height		equ $BA					;
tile			equ $BC					;
;charH		equ $BE					;
yOffsetL		equ $C0					;
yOffsetH		equ $C2					;
											;
grfxBuffer	equ $B2 ; (20 bytes)	;
;---------------------------------

;---------------------------------
temp4			equ $C6 					;
linecolor	equ $C6 					;
;---------------------------------

temp 			equ $C8
temp2			equ $C9

frame 		equ $CA

locX			equ $CB
locY			equ $CC

joystick		equ $CD	; for debouncing joystick

mode			equ $CF	; main mode control
mode_cnt		equ $D0	; main mode counter


bkcolor		equ $D1

backChar		equ $D2

skipBlank	equ $D3

textColor	equ $D4
;---------------------------------------------------------------------------
;---------------------------------------------------------------------------
	org $F000
;---------------------------------------------------------------------------
;---------------------------------------------------------------------------

swap
	byte $00,$10,$20,$30, $40,$50,$60,$70, $80,$90,$a0,$b0, $c0,$d0,$e0,$f0
	byte $01,$11,$21,$31, $41,$51,$61,$71, $81,$91,$a1,$b1, $c1,$d1,$e1,$f1
	byte $02,$12,$22,$32, $42,$52,$62,$72, $82,$92,$a2,$b2, $c2,$d2,$e2,$f2
	byte $03,$13,$23,$33, $43,$53,$63,$73, $83,$93,$a3,$b3, $c3,$d3,$e3,$f3
	byte $04,$14,$24,$34, $44,$54,$64,$74, $84,$94,$a4,$b4, $c4,$d4,$e4,$f4
	byte $05,$15,$25,$35, $45,$55,$65,$75, $85,$95,$a5,$b5, $c5,$d5,$e5,$f5
	byte $06,$16,$26,$36, $46,$56,$66,$76, $86,$96,$a6,$b6, $c6,$d6,$e6,$f6
	byte $07,$17,$27,$37, $47,$57,$67,$77, $87,$97,$a7,$b7, $c7,$d7,$e7,$f7
	byte $08,$18,$28,$38, $48,$58,$68,$78, $88,$98,$a8,$b8, $c8,$d8,$e8,$f8
	byte $09,$19,$29,$39, $49,$59,$69,$79, $89,$99,$a9,$b9, $c9,$d9,$e9,$f9
	byte $0a,$1a,$2a,$3a, $4a,$5a,$6a,$7a, $8a,$9a,$aa,$ba, $ca,$da,$ea,$fa
	byte $0b,$1b,$2b,$3b, $4b,$5b,$6b,$7b, $8b,$9b,$ab,$bb, $cb,$db,$eb,$fb
	byte $0c,$1c,$2c,$3c, $4c,$5c,$6c,$7c, $8c,$9c,$ac,$bc, $cc,$dc,$ec,$fc
	byte $0d,$1d,$2d,$3d, $4d,$5d,$6d,$7d, $8d,$9d,$ad,$bd, $cd,$dd,$ed,$fd
	byte $0e,$1e,$2e,$3e, $4e,$5e,$6e,$7e, $8e,$9e,$ae,$be, $ce,$de,$ee,$fe
	byte $0f,$1f,$2f,$3f, $4f,$5f,$6f,$7f, $8f,$9f,$af,$bf, $cf,$df,$ef,$ff

pf0shading
	byte 0,$C3,0,$C3, 0,0,$C3,$C3, 0,0,0,0, $C3,$C3,0,0
	byte $3C,$FF,$3C,$FF, $3C,$3C,$FF,$FF, $3C,$3C,$3C,$3C, $FF,$FF,$3C,$3C
	byte 0,$C3,0,$C3, 0,0,$C3,$C3, 0,0,0,0, $C3,$C3,0,0
	byte $3C,$FF,$3C,$FF, $3C,$3C,$FF,$FF, $3C,$3C,$3C,$3C, $FF,$FF,$3C,$3C

	byte 0,$C3,0,$C3, 0,0,$C3,$C3, 0,0,0,0, $C3,$C3,0,0
	byte 0,$C3,0,$C3, 0,0,$C3,$C3, 0,0,0,0, $C3,$C3,0,0
	byte $3C,$FF,$3C,$FF, $3C,$3C,$FF,$FF, $3C,$3C,$3C,$3C, $FF,$FF,$3C,$3C
	byte $3C,$FF,$3C,$FF, $3C,$3C,$FF,$FF, $3C,$3C,$3C,$3C, $FF,$FF,$3C,$3C

	byte 0,$C3,0,$C3, 0,0,$C3,$C3, 0,0,0,0, $C3,$C3,0,0
	byte 0,$C3,0,$C3, 0,0,$C3,$C3, 0,0,0,0, $C3,$C3,0,0
	byte 0,$C3,0,$C3, 0,0,$C3,$C3, 0,0,0,0, $C3,$C3,0,0
	byte 0,$C3,0,$C3, 0,0,$C3,$C3, 0,0,0,0, $C3,$C3,0,0

	byte $3C,$FF,$3C,$FF, $3C,$3C,$FF,$FF, $3C,$3C,$3C,$3C, $FF,$FF,$3C,$3C
	byte $3C,$FF,$3C,$FF, $3C,$3C,$FF,$FF, $3C,$3C,$3C,$3C, $FF,$FF,$3C,$3C
	byte 0,$C3,0,$C3, 0,0,$C3,$C3, 0,0,0,0, $C3,$C3,0,0
	byte 0,$C3,0,$C3, 0,0,$C3,$C3, 0,0,0,0, $C3,$C3,0,0

	;1,3=water 7=wall 8=door 9,A=rock
pfcolors
	byte 0,$80,0,$80, 0,0,$F0,$F0, 0,0,0,0, $02,$02,0,0
	byte $80,$80,$80,$80, $80,$80,$80,$80, $80,$80,$80,$80, $80,$80,$80,$80
	byte 0,$80,0,$80, 0,0,$F0,$F0, 0,0,0,0, $02,$02,0,0
	byte $80,$80,$80,$80, $80,$80,$80,$80, $80,$80,$80,$80, $80,$80,$80,$80

	byte 0,$80,0,$80, 0,0,$F0,$F0, 0,0,0,0, $02,$02,0,0
	byte 0,$80,0,$80, 0,0,$F0,$F0, 0,0,0,0, $02,$02,0,0
	byte $F0,$F0,$F0,$F0,$F0,$F0,$F0,$F0,$F0,$F0,$F0,$F0,$F0,$F0,$F0,$F0
	byte $F0,$F0,$F0,$F0,$F0,$F0,$F0,$F0,$F0,$F0,$F0,$F0,$F0,$F0,$F0,$F0

	byte 0,$80,0,$80, 0,0,$F0,$F0, 0,0,0,0, $02,$02,0,0
	byte 0,$80,0,$80, 0,0,$F0,$F0, 0,0,0,0, $02,$02,0,0
	byte 0,$80,0,$80, 0,0,$F0,$F0, 0,0,0,0, $02,$02,0,0
	byte 0,$80,0,$80, 0,0,$F0,$F0, 0,0,0,0, $02,$02,0,0

	byte $02,$02,$02,$02,$02,$02,$02,$02,$02,$02,$02,$02,$02,$02,$02,$02
	byte $02,$02,$02,$02,$02,$02,$02,$02,$02,$02,$02,$02,$02,$02,$02,$02
	byte 0,$80,0,$80, 0,0,$F0,$F0, 0,0,0,0, $02,$02,0,0
	byte 0,$80,0,$80, 0,0,$F0,$F0, 0,0,0,0, $02,$02,0,0


;---------------------------------------------------------------------------
; println
;---------------------------------------------------------------------------
; New version of David Cranes flicker text from Stellar Track/Basic Programming
;---------------------------------------------------------------------------


; grfxBuffer offsets
7; x     x     x     x     x
;00 02 04 06 08 10 12 14 16 18
; A  B  C  D  E  F  G  H  I  J
;    x     x     x     x     x

println
;	ldx #0
;	stx frame

	sec

	ldy #$90						;2

	lax (grfxBuffer + 18),y	;5

	lda frame
	and #1
	beq noShift

	lax (grfxBuffer + 16),y	;5

	lda #$80
	sta HMP0
	sta HMP1

	sta WSYNC
	sta HMOVE
	sta HMCLR
	jmp frameB

	bne frameA
noShift
	sta WSYNC
	sta HMCLR
	sta $2D

frameA ;@3
	stx.w GRP0						;3
	jmp enterA

loopA ;@21
	lda (grfxBuffer + 18),y	;5 @3
	sta GRP0						;3
enterA
	lda (grfxBuffer + 14),y	;5
	sta.w GRP1						;3

	sta HMCLR

	ldx linecolor				;3 @30

	lda (grfxBuffer + 10),y	;5
	stx COLUPF					;3
	
	sta.w GRP0						;3
	lda (grfxBuffer + 6),y	;5
	sta GRP1						;3 @49

	lda (grfxBuffer + 2),y	;5
	sta GRP0						;3
	lda bkcolor					;3 @59
	sta COLUPF					;3 @66

	ldx #$80						;2
	stx HMP0						;3 
	stx HMP1						;3 @72

	tya							;2
	sbc #$10
	tay

	sta HMOVE					;3 @3

	;--------------------------

	lda (grfxBuffer + 16),y	;5
	sta GRP0						;3
	lda (grfxBuffer + 12),y	;5
	sta.w GRP1					;3 @20

	nop							;2
	nop							;2

	ldx.w linecolor			;3 @30
	lda (grfxBuffer + 8),y	;5
	stx COLUPF					;3
	ldx #$80						;2
	sta GRP0						;3
	lda (grfxBuffer + 4),y	;5
	sta GRP1						;3 @49

	lda (grfxBuffer + 0),y	;5
	sta GRP0						;3
	lda bkcolor					;2 @59
	sta COLUPF					;3 @66
	sta HMCLR					;3 
	stx HMM0
	tya
	sta HMOVE					;3 @74
	sbc #$10
	;-------------------------
	tay
	bcs loopA					;2/3

quitA
	lda #0
	sta GRP0
	sta GRP1
	rts	


	;====================================
	;====================================
	;====================================

frameB ;@7
	lda (grfxBuffer + 12),y	;5
	sta GRP1						;3
	stx GRP0						;3
	nop
	nop
	;sta $2D
;	dec $2D						;5 delay
loopB ;@24
	ldx linecolor				;3 @27

	lda (grfxBuffer + 8),y	;5
	stx.w COLUPF				;3
	ldx #$80						;2
	sta GRP0						;3
	lda (grfxBuffer + 4),y	;5
	sta GRP1						;3 @46

	lda (grfxBuffer + 0),y	;5
	sta GRP0						;3
	lda bkcolor					;2 @56
	sta COLUPF					;3 @63
	stx HMCLR					;3 
	stx HMM0						;3
	tya							;2
	sta HMOVE					;3 @74

	;--------------------------
	sta HMCLR
	sbc #$10					;2 @0
	tay
	lda (grfxBuffer + 18),y	;5
	sta.w GRP0						;3
	lda (grfxBuffer + 14),y	;5
	sta GRP1						;3 @21

	stx HMP0					;4

	ldx linecolor			;3 @30
	lda (grfxBuffer + 10),y	;5
	stx COLUPF					;3
	sta GRP0						;3
	lda (grfxBuffer + 6),y	;5
	sta GRP1						;3 @51

	lda (grfxBuffer + 2),y	;5
	sta GRP0						;3
	lda bkcolor					;2 @61
	sta.w COLUPF					;3 @66
	ldx #$80							;2
	tya
	sbc #$10						;2

	bcc quitB					;2/3
	stx HMP1						;3 @69
	
	sta WSYNC					;3 @0
	sta HMOVE					;3 @3

	tay

	lda (grfxBuffer + 16),y	;5
	sta GRP0						;3
	lda (grfxBuffer + 12),y	;5
	sta GRP1						;3 @19

	jmp loopB
	bpl loopB					;2/3

quitB
	sta WSYNC
	jmp quitA



	align 256
	; Character Font
charData
	;include rpgfont.h 
	include tiles1.h

				
;---------------------------------------------------------------------------
; Start of Program
;---------------------------------------------------------------------------
; Clear memory, locate character graphics positions and such,
; initialize key memory values, start GameLoop.
;-------------------------------------------------------------
Start
	sei  	
	cld  		
	ldx #$FF
	txs  		
	lda #0
clear   
	sta 0,x
	dex
	bne clear	

;---------------------------------------------------------------------------
; Initialize variables / registers
;---------------------------------------------------------------------------

	;lda #255 ; Solid PF for debugging
	lda #%11001100
	sta PF0
	sta PF2
	lda #%00110011
	sta PF1

	; Set background color (remove later)
	lda #$C0
;	lda #0
	sta bkcolor

	; Set players to grey
	lda #PLAYERCOLOR
	sta COLUP0
	sta COLUP1

	lda #STARTPOSX
	sta locX
	lda #STARTPOSY
	sta locY
	
	lda #M_LOAD
	sta mode
	lda #C_LOAD
	sta mode_cnt

;	lda #5
;	sta status	
	

	jsr TextSetup

	

;	jsr loadRoom

	;-----------------------------------------------------
	;the entire font is limited to one page, so we can hard code that page
	;value into the MSB position of our graphics buffer
	;-----------------------------------------------------
	ldx #23         ;graphics buffer is 24 bytes
	lda #>charData   ;load up the page value for the font
grOff
	sta grfxBuffer,x        ;stuff it, won't ever change
	dex             ;back up two bytes
	dex
	bpl grOff       ;and loop

	lda #0
	sta CTRLPF

;--------------------------------------------------------------------------
; GameLoop
;--------------------------------------------------------------------------
GameLoop
	jmp VSync 	;start vertical retrace
rtnVSync

	lda mode
	cmp #M_LOAD
	beq mapLoader

	jmp VBlank    	; spare time during screen blank
rtnVBlank

	jmp Picture		; draw one screen
rtnPicture

	jmp overscan	; do overscan
rtnOverscan

	jmp GameLoop    ;back to top



;--------------------------------------------------------------------------
; mapLoader
;--------------------------------------------------------------------------
; Fully loads the current 10x10 location into
; the tile buffer.  This takes too much time to do during
; VBlank or overscan, so the screen is blanked during this load.
; This runs during what would normally draw the TV picture.
; The load takes 11 frames.  It loads the full screen simply
; by clearing the screen and then using the scroll load 10 times
; to load the whole pictures.  The Y location can be anywhere in the
; map, but the low nibble (screen offset) of the X location must be zero.
;--------------------------------------------------------------------------
mapLoader
	lda INTIM		;check timer for end of VBLANK period
	bne mapLoader	;loop until it reaches 0

	lda #$80
	sta WSYNC
	sta VBLANK  	;end screen blank

	; Wait through the visible picture and overscan
	lda #VBLANKOVERSCANTIME
	sta T1024T

	ldx #7
aFewLines
	sta WSYNC
	lda mode_cnt
	sta COLUPF
	sta PF0
	sta PF1
	sta PF2
	dex
	bne aFewLines
	lda #0
	sta COLUPF


	dec mode_cnt
	ldy mode_cnt

	cpy #10
	bne noClear
	lda #$00
	ldx #50
CopyLoop
	sta levelDataA,x
	dex
	bpl CopyLoop

	sed
	lda locY
	clc
	adc #$10
	sta locY
	cld

	jmp loadMore

noClear

	lda #M_SCRU
	sta mode

	sed
	lda locY
	sec
	sbc #$01
	sta locY
	cld

	ldy #0
	jsr loadRoom

	lda #M_LOAD
	sta mode


	lda mode_cnt
	bne loadMore	
	lda #M_MAP
	sta mode
	jmp endMapLoop
loadMore
	jsr scrollDown

endMapLoop
	lda INTIM	;check timer for end of VBLANK period
	bne endMapLoop	;loop until it reaches 0

	ldx #7
aFewMore
	sta WSYNC
	dex
	bne aFewMore

	jmp GameLoop




;--------------------------------------------------------------------------
; getRoom
;--------------------------------------------------------------------------
; For debugging: load an example map into 
; the tile buffer.
;--------------------------------------------------------------------------
;getRoom
;	ldx #50
;CopyLoopST
;	lda levelA,x
;	sta levelDataA,x
;	dex
;	bpl CopyLoopST
;	rts



;--------------------------------------
; TextSetup
;--------------------------------------
; use cycle counting to get the GRP 
; registers configured and located
; in exactly the right spot
;--------------------------------------
TextSetup
	sta WSYNC       ;newline
	sta RESM1
	lda #$36        ;triple copy sprites, spaced wide
	sta NUSIZ0      ;for both player graphic registers
	lda #$32
	sta NUSIZ1
	lda #%01000000
	sta HMM0
	nop
	nop
	sta temp
	sta $2D
	sta $2D
	sta RESP0
	lda #$F0
	sta RESP1
	sta HMP0
	lda #$E0
	sta HMP1
	dec $2D
	dec $2D
	nop
	nop
	
	sta RESM0

	sta WSYNC       ;newline
	sta HMOVE       ;anchor them down
	rts



;--------------------------------------------------------------------------
; VSync
;--------------------------------------------------------------------------
VSync
	lda #$02
	sta WSYNC	; End last scanline
;	lda #$82
	sta VBLANK
	lda #2		;bit 1 needs to be 1 to start retrace
	sta VSYNC	;start retrace
	sta WSYNC 	;wait a few lines
	sta WSYNC 
	lda #44		;prepare timer to exit blank period (44)
	sta TIM64T	;turn it on
	sta WSYNC 	;wait one more
	sta VSYNC 	;done with retrace, write 0 to bit 1

	jmp rtnVSync ; VSync





roomX
	byte 0,1,0,1
roomY
	byte 0,0,1,1
roomX10
	byte 0,$10,0,$10
roomY10
	byte 0,0,$10,$10


;--------------------------------------------------------------------------
; loadRoom
;--------------------------------------------------------------------------
; Set Y to screen cell (0-3) right before loadRoom is called.
; Note that this function uses the stack pointer and assumes that the
; JSR was called with nothing on the stack.
;--------------------------------------------------------------------------
loadRoom

	sty temp2

	;--------------------------------
	; Get pointer to level definition

	; Get pointer to row
	ldx locX

	lda swap,x
	and #$0F

	clc
	adc roomX,y
	tax
	lda locY
	lsr
	lsr
	lsr
	lsr
	clc
	adc roomY,y
	clc
	adc RowPointer,x
	tay

	; Check for overstepping end of level data
	cpy levelSize
	bmi inLevelData
	rts

inLevelData
	; Get the pointer to the current screen definition
	ldx ScreenPointer,y
	; Get pointer to next so we'll know
	; when we reach the end of the list.
	iny
	lda ScreenPointer,y
	sta temp4

	;-------------------------------
	; Process level definition codes

tileLoop

	sed
	; Calculate Y position	
	lda locY
	and #$0F
	sta temp3
	lda LevelData,x
	and #$0F
	cmp #$0A	; check for onebyte setup
	bmi skipOneByteA
	lda #0
skipOneByteA	
	sec
	sbc temp3
	ldy temp2
	clc
	adc roomY10,y
	sta startY
	stx temp3

;	cld

	; Check for 1 byte command
	lda LevelData,x
	tay
	and #$0F
	cmp #$0A
	bpl oneByteL
	tya
	clc
	adc #$00
	bcs oneByteH
	bcc twoByte

oneByteL
	tya
	and #$0F
	sec
	sbc #$0A
	tay
	lda oneByteTile,y
	sta tile
	lda oneByteSize,y
	sta width
	lda #10
	sta height
	jmp fromOneByte

oneByteH	
	lda swap,y
	and #$0F
	sec
	sbc #10
	tay
	lda oneByteTile,y
	sta tile
	lda #10
	sta width
	lda oneByteSize,y
	sta height
	jmp fromOneByte


twoByte
	inc temp3

	; Get current tile
	lda LevelData+1,x
	and #%10000111
	bpl notSpecial

	; Handle Special Tiles
	; (random tree generation, monsters, events, warps)
	lda #%10001000
	sta tile
	bne fromSpecial

notSpecial
	tay
	lda tileType,y
	sta tile


fromSpecial

	; Set up block size
	lda LevelData+1,x
	lsr
	lsr
	lsr
	and #$0F
	tay
	lda blockX,y
	sta width
	lda blockY,y
	sta height

fromOneByte

	; Calculate X position	
	lda locX
	and #$0F
	sta temp ;temp3
	ldy LevelData,x
;	ldy #0
	lda swap,y
	and #$0F
	cmp #$0A	; check for onebyte setup
	bmi skipOneByteB
	lda #0
skipOneByteB
	sec
	sbc temp ;temp3
	ldx temp2
	clc
	adc roomX10,x
	; save tile def pointer
	sta startX
	clc
	adc width
	sta width
;	ldx temp3

	sed

;	cld

	; -------------------------
	; check for scrolling modes
	; where only 1 row/column
	; is rendered
	; -------------------------
	lda mode
	and #%00000011
	cmp #M_SCRL
	beq leftScr
	cmp #M_SCRR
	beq rightScr
	cmp #M_SCRU
	beq upScr
	cmp #M_SCRD
	beq downScr
	jmp yLoop

leftScr
	lda startX
	cmp #10
	bpl goSkip
	lda width
	cmp #10
	bmi goSkip
	ldx #9
	stx startX
	lda #$10
	sta width
	bne xLoop

rightScr
	lda startX
	cmp #1
	bpl goSkip
	lda width
	cmp #1
	bmi goSkip
	ldx #0
	stx startX
	lda #1
	sta width
	bne xLoop

VBLANKMARGIN equ 5

upScr
	lda startY
	cmp #1
	bpl goSkip
	clc
	adc height
	cmp #1
	bmi goSkip
	ldy #0
	sty startY
	lda #1
	sta height
	bne yLoop
			
downScr
	lda startY
	cmp #$10
	bpl goSkip
	clc
	adc height
	cmp #$10
	bmi goSkip
	ldy #9
	sty startY
	lda #1
	sta height
	bne yLoop

goSkip
	ldy #$88
	; Make sure we're not almost out of time
	lda INTIM
	sbc #VBLANKMARGIN
	bmi pictureLoop2
;	lda skipBlank
;	beq pictureLoop2
	jmp skipBlock

			
	; ------------------------------------------------
	; Almost to the end of VBlank, go ahead and end it
	; And start using the blank area at the top of the
	; screen before the map area.
pictureLoop2
	lda INTIM	;check timer for end of VBLANK period
	bne pictureLoop2	;loop until it reaches 0

	lda #$80
	sta WSYNC
	sta VBLANK  	;end screen blank

	lda #TOPBLANKTIME
	sta TIM64T
	sta skipBlank

	cpy #$88
	bne backFromVBlank
	jmp skipBlock
			
	;---------------------------
	; Main loop for filling area
yLoop
	ldx startX
xLoop
	ldy startY

	; Make sure we're not almost out of time
	lda INTIM
	sbc #VBLANKMARGIN
	bmi pictureLoop2
;	lda skipBlank
;	beq pictureLoop2
backFromVBlank
	

	; ------------------------
	; Check for out of range
	tya
	and #$F0
	bne skipChar2
	txa
	cmp #$10
	bpl skipChar2
	and #$F0
	bne skipChar

	lda XPosition,x
	cld
	clc
	adc YPosition,y
	sed
	tay

	; Check for a randomized tile
	lda tile
	bpl notRandomTile
	and #%01000000
	bne notRandomTile

	txs
	txa
;	sec
	adc locX
	asl
	asl
	adc locY
	adc startY
	tax
	lda $F400,x
	tsx

	lsr
	and #%00000001
	beq fromRandom
	bcs drawTreeB

drawTreeA
	txa
	and #1
	beq highNib3
	lda #$50
	byte $2C
highNib3
	lda #$05
	jmp fromRandom

drawTreeB
	tsx
	txa
	and #1
	beq highNib4
	lda #$40
	byte $2C
highNib4
	lda #$04
	jmp fromRandom
	
notRandomTile
	txa
	and #1
	beq highNib2
	lda #$F0
	byte $2C ; BIT absolute -- skip next two bytes
highNib2
	lda #$0F
lowNib2
	and tile

fromRandom
	ora levelDataA,y
	sta levelDataA,y


skipChar

	txa
	clc
	adc #1
	tax
	cpx width
	bne xLoop 
skipChar2
	lda startY
	clc
	adc #1
	sta startY
	dec height
	beq stopYLoop
	jmp yLoop	
stopYLoop
	;---------------------------
	; See if there are any more 
	; blocks to process
skipBlock
	ldx temp3
;	inx
	inx
	cpx temp4
	beq quitLoop
	jmp tileLoop
quitLoop

	cld

	ldx #253
	txs

	rts


randomArray
	byte 56,72,243,123, 74,198,2,34, 203,172,8,129, 192,37,198,23


	
XPosition
	byte 4,0,0,1,1, 2,2,3,3,4
YPosition
	byte 0,5,10,15,20, 25,30,35,40,45



;--------------------------------------------------------------------------
; scrollDown
;--------------------------------------------------------------------------
scrollDown
	ldy #44
sDnLoop
	lda levelDataA,y
	sta levelDataA+5,y
	dey
	bpl sDnLoop

	lda #0
	sta levelDataA
	sta levelDataA+1
	sta levelDataA+2
	sta levelDataA+3
	sta levelDataA+4
	rts

;--------------------------------------------------------------------------
; scrollUp
;--------------------------------------------------------------------------
scrollUp
	ldy #0
sUpLoop
	lda levelDataA+5,y
	sta levelDataA,y
	iny
	cpy #45
	bne sUpLoop

	lda #0
	sta levelDataA+45
	sta levelDataA+46
	sta levelDataA+47
	sta levelDataA+48
	sta levelDataA+49
	rts


;--------------------------------------------------------------------------
; scrollRight
;--------------------------------------------------------------------------
scrollRight
	lda #45
	sec
shiftLLoop
	tay
	
	lda levelDataA+4,y
	and #$F0
	sta temp
	ldx levelDataA,y
	lda swap,x				;4
	and #$0F
	ora temp
	sta levelDataA+4,y

	lda swap,x
	and #$F0
	sta temp
	ldx levelDataA+1,y
	lda swap,x				;4
	and #$0F
	ora temp
	sta levelDataA,y

	lda swap,x
	and #$F0
	sta temp
	ldx levelDataA+2,y
	lda swap,x				;4
	and #$0F
	ora temp
	sta levelDataA+1,y

	lda swap,x
	and #$F0
	sta temp
	ldx levelDataA+3,y
	lda swap,x				;4
	and #$0F
	ora temp
	sta levelDataA+2,y

	lda swap,x
	and #$F0
	sta temp
	ldx levelDataA+4,y
	lda swap,x
	and #$0F
	ora temp
	sta levelDataA+3,y

	tya
	sbc #5
	bcs shiftLLoop

	ldy #9
	lda #0
	sec
clearSideR
	ldx YPosition,y
	lda levelDataA+4,x
	and #$0F
	sta levelDataA+4,x
	dey
	bpl clearSideR

	rts

;--------------------------------------------------------------------------
; scrollLeft
;--------------------------------------------------------------------------
scrollLeft

	lda #45
	sec
shiftRLoop
	tay
	
	lda levelDataA+4,y
	and #$0F
	sta temp
	ldx levelDataA+3,y
	lda swap,x
	and #$F0
	ora temp
	sta levelDataA+4,y

	lda swap,x
	and #$0F
	sta temp
	ldx levelDataA+2,y
	lda swap,x
	and #$F0
	ora temp
	sta levelDataA+3,y

	lda swap,x
	and #$0F
	sta temp
	ldx levelDataA+1,y
	lda swap,x
	and #$F0
	ora temp
	sta levelDataA+2,y

	lda swap,x
	and #$0F
	sta temp
	ldx levelDataA,y
	lda swap,x
	and #$F0
	ora temp
	sta levelDataA+1,y

	lda swap,x
	and #$0F
	sta temp
	ldx levelDataA+4,y
	lda swap,x
	and #$F0
	ora temp
	sta levelDataA,y

	tya
	sbc #5
	bcs shiftRLoop

	ldy #9
	lda #0
	sec
clearSideL
	ldx YPosition,y
	lda levelDataA+4,x
	and #$F0
	sta levelDataA+4,x
	dey
	bpl clearSideL

	rts
	
	


;--------------------------------------------------------------------------
; Overscan
;--------------------------------------------------------------------------
; More Game Logic
;--------------------------------------------------------------------------
overscan

	sta WSYNC	

	lda #36		; Use the timer to make sure overscan takes (34)
	sta TIM64T	; 30 scan lines.  29 scan lines * 76 = 2204 / 64 = 34.4

	; Erase your character
	lda backChar
	sta levelDataA+MANOFFSET

	; Only do overscan processing
	; if in MAP mode.
	lda mode
	cmp #M_MAP
	beq goOS
	jmp endOS
goOS


	; *****************
	; Joystick
	; *****************

	lda locX
	sta temp
	lda locY
	sta temp2

	sed
	clc
	lda joystick
	bne notUp
	lda SWCHA
	tax
	bmi notRight
	lda locX
	cmp #MAPSIZEX
	clc
	beq notRight
	adc #1
	sta locX
	cld
	jsr scrollRight
	lda locX
	and #$0F
	beq specialLeft
	lda #M_SCRL
	sta mode
	bpl endJoy
specialLeft
	lda #M_SCRLS
	sta mode
	bpl endJoy
notRight
	clc
	txa
	and #%01000000
	bne notLeft
	lda locX
	beq notLeft
	sbc #0
	sta locX
	cld
	jsr scrollLeft
	lda #M_SCRR
	sta mode
	bpl endJoy
notLeft
	txa
	and #%00100000
	bne notDown
	lda locY
	cmp #MAPSIZEY
	clc
	beq notDown
	adc #1
	sta locY
	cld
	jsr scrollUp
	lda locY
	and #$0F
	beq specialDown
	lda #M_SCRD
	sta mode
	bpl endJoy
specialDown
	lda #M_SCRDS
	sta mode
	bpl endJoy
notDown
	txa
	and #%00010000
	bne notUp
	lda locY
	beq notUp
	sbc #0
	sta locY
	cld
	jsr scrollDown
	lda #M_SCRU
	sta mode
	bpl endJoy
notUp
	cld

endJoy
	lda #C_SCR
	sta mode_cnt

	; De-bounce
	lda SWCHA
	eor #$FF
	sta joystick

;	lda frame
;	and #%00000011
;	bne notJoyPulse
;	sta joystick
;notJoyPulse

	; Joystick button reloads entire screen 
	; (for debugging)
	lda INPT4
	bmi noButton
;	jsr getRoom
	lda #M_LOAD
	sta mode
	lda #C_LOAD
	sta mode_cnt
noButton

;	sta WSYNC
;	sta WSYNC


endOS	
	lda INTIM	; We finished, but wait for timer
	bne endOS	; by looping till zero
	

	jmp rtnOverscan


	; order L,R,U,D, LS,RSx,US,DSx
scrollScreenA
	byte 1,0,0,2, 0,1,2,0
scrollScreenB
	byte 3,2,1,3, 2,3,3,1

;--------------------------------------------------------------------------
; VBlank
;--------------------------------------------------------------------------
; Game Logic
;--------------------------------------------------------------------------
VBlank
	inc frame	;count frame

	lda #0
	sta skipBlank

	; Check if we're scrolling...
	lax mode
	and #SCROLL_MASK
	bne noScrollLoad
	; then load new strip of tiles.
	txa
	and #~SCROLL_MASK
;	and #%00000111
	tax
	dec mode_cnt
	beq L2
	ldy scrollScreenA,x
	jsr loadRoom
	jmp skipWater
L2
	ldy scrollScreenB,x
	jsr loadRoom
	lda #M_MAP
	sta mode
	jmp skipWater

noScrollLoad

	; *****************
	; Water Animation
	; *****************
	lda frame
	and #%00001111
	bne noChange

	ldy #50
WaterLoop
	ldx levelDataA,y
	txa
	and #%11010000
;	(xxa)
;	byte $8b, %11010000
	cmp #$10
	bne noWater1
	txa
	eor #%00100000
	tax
noWater1
	txa
	and #%00001101
;  (xxa)
;	byte $8b, %00001101
	cmp #$01
	bne noWater2
	txa
	eor #%00000010
	tax
noWater2
	txa
	stx levelDataA,y
	dey
	bpl WaterLoop
noChange
skipWater

	; Draw your character
	lda levelDataA+MANOFFSET
	sta backChar
;	lda frame
;	and #%00011000
;	beq noMan
	lda #$F0
	ora levelDataA+MANOFFSET
	sta levelDataA+MANOFFSET

noMan

;	sta WSYNC
;	sta WSYNC
;	sta WSYNC
;	sta WSYNC
;	sta WSYNC
;	sta WSYNC

	jmp rtnVBlank






;--------------------------------------------------------------------------
; Draw TV Pictures
;--------------------------------------------------------------------------
;
;--------------------------------------------------------------------------
Picture

	sta HMCLR

	lda skipBlank
	bne topBlankLoop

pictureLoop
	lda INTIM	;check timer for end of VBLANK period
	bne pictureLoop	;loop until it reaches 0

	lda #$80
	sta WSYNC
	sta VBLANK  	;end screen blank
	
	; Wait 18 more lines
	lda #TOPBLANKTIME
	sta TIM64T

topBlankLoop
	lda INTIM	;check timer for end of VBLANK period
	bne topBlankLoop	;loop until it reaches 0
	sta WSYNC

;---------------------------------------------------------------------------
;	ldx #18
;ScanLoop3:
;	sta WSYNC
;	dex
;	bne ScanLoop3

	lda #$08
	sta COLUPF
	lda #255
	sta ENAM0
	sta PF1
	sta PF2
	sta PF0

	sta WSYNC
	sta WSYNC
	sta WSYNC
	lda bkcolor
	sta COLUPF
	sta COLUBK

	lda #0
	sta PF1
	sta PF0
	sta PF2

	;--------------------------
ScanLoop:
;	ldx #28
;	stx TIM8T

;	ldy linecolor			;3
;	sty COLUPF				;3

	tay

	lax levelDataA+1,y
	and #$0f  
	sta grfxBuffer+10
	txa
	lsr
	lsr
	lsr
	lsr
	sta grfxBuffer+12


	lda pf0shading,x
	and #$F0
	sta temp				; 3 extra

	ldx levelDataA,y
	lda pf0shading,x
;	and #$0F
	lsr
	lsr
	lsr
	lsr
	ora temp
	sta PF2

;	lda bkcolor			;3
;	sta COLUPF				;3

	lax levelDataA,y
	and #$0f  
	sta grfxBuffer+14
	txa
	lsr
	lsr
	lsr
	lsr
	sta grfxBuffer+16



	; Determine color of PF for current line
	; based on tiles present in line
	lda pfcolors,x			; 4     
	ldx levelDataA+1,y	; 4
	ora pfcolors,x			; 4
	ldx levelDataA+3,y	; 4
	ora pfcolors,x			; 4
	ldx levelDataA+4,y	; 4
	ora pfcolors,x			; 4
	ldx levelDataA+2,y	;   needed anyway
	ora pfcolors,x			; 4
	sta linecolor			; 3 = 35 (instead of 41)

	txa
	and #$0f  
	sta grfxBuffer+6
	txa
	lsr
	lsr
	lsr
	lsr
	sta grfxBuffer+8

	lda pf0shading,x
	and #$F0				; 2 extra
	sta PF0

	lax levelDataA+3,y
	and #$0f
	sta grfxBuffer+2
	txa
	lsr
	lsr
	lsr
	lsr
	sta grfxBuffer+4

	lda pf0shading,x
	asl
	asl
	asl
	asl
	sta temp

	ldx levelDataA+4,y
	lda pf0shading,x
	and #$0F				; 2 extra
	ora temp				; 3 extra
	sta PF1

	txa
	and #$0f  
	sta grfxBuffer+18
	txa
	lsr
	lsr
	lsr
	lsr
	sta grfxBuffer+0

;	nop
;	nop
;	nop
;	nop
;	nop
;	nop
;	nop

	sty temp2
	
textTimer
;	ldy INTIM
;	bne textTimer

	jsr println
	lda temp2
	
	clc

	adc #$05
	cmp #$32

	beq quitScanLoop
	jmp ScanLoop
quitScanLoop


	sta WSYNC
	sta WSYNC
	lda #$08
	sta COLUPF
	lda #255
	sta PF0
	sta PF1
	sta PF2	
	sta WSYNC
	sta WSYNC
	sta WSYNC
	lda #0
	sta ENAM0
	sta COLUPF
	sta COLUBK

	ldx #26
ScanLoop2:
	sta WSYNC
	dex
	bne ScanLoop2

	lda #0
	sta COLUBK
	sta COLUPF

	jmp rtnPicture





	; Level Data

levelA
	byte $90,$00,$00,$00,$09
	byte $87,$06,$50,$06,$07
	byte $09,$00,$00,$00,$90
	byte $56,$00,$00,$00,$00
	byte $00,$05,$00,$01,$10

	byte $06,$00,$00,$11,$10
	byte $55,$50,$00,$11,$10
	byte $50,$65,$01,$11,$10
	byte $05,$01,$11,$11,$15
	byte $11,$11,$11,$11,$11

; second screen
;	byte $9a,$a9,$9a,$99,$aa
;	byte $00,$00,$00,$0a,$90
;	byte $99,$00,$00,$09,$aa
;	byte $9a,$a9,$09,$9a,$9a
;	byte $11,$11,$b1,$11,$11

;	byte $11,$11,$b1,$11,$11
;	byte $90,$00,$09,$aa,$9a
;	byte $9a,$90,$00,$a9,$a9
;	byte $aa,$00,$00,$a9,$9a
;	byte $9a,$00,$0a,$9a,$9a


;times10
;	byte 0,10,20,30,40,50,60,70,80,90,100
;	byte 100,110,120,130,140,150,160,170,180,190,200


;levelBk
;	byte $C0
;	byte $e0


;--------------------------------------------------------------------------
; Level Data
;--------------------------------------------------------------------------
; [ Demo will be a 6x6 area (60x60 cells) ]
;
; Tiles
; -----
; 0 0000 Grass
; 1 0001 WaterA
; 2 0010 Gate
; 3 0010 WaterB
;
; 4 0100 TreeA
; 5 0101 TreeB
; 6 0110 RockA
; 7 0111 RockB
; 
; 8 1000 MonsterA
; 9 1001 MonsterB
; a 1010 MonsterC
; b 1011 MonsterD
;
; c 1100 Wall
; d 1101 Door
; e 1110 Hut
; f 1111 Homestar
;
; Two Byte Codes
; --------------
; First byte is X/Y coordinates (from left of 10x10 area)
; Second byte = object code:
; %0 0000 000
;  | \  / \ /
;  |  \/   \|
;  |   |    |_Object type
;  |   |
;  |   |_Shape
;  |
;  |_Special
;
; Object Type
; 000 = Water
; 001 = Rock
; 010 = Wall
; 011 = Door
; 100 = TreeA
; 101 = TreeB
; 110 = Gate
; 111 = Hut
;
; Special
; 000 = Tree sparse
; 001 = Tree thick
; 010 = Rock sparse
; 011 = Event
; 100 = Monster A
; 101 = Monster B
; 110 = Monster C
; 111 = Monster D
;
; Single Byte Codes
; -----------------
; 
; %0000 0000
;  Code Xpos
;   - or -
; %0000 0000
;  Ypos Code
;
; $A full row or column water
; $B double row or column water
; $C full row or column rocks
; $D double row or column rocks
; $E 3x row of sparse trees
; $F 5x row of sparse trees

oneByteTile
	byte $11,$11,$99,$99,$55,$11

oneByteSize
	byte 1,2,3,5,3,10



blockX
	byte 1,2,1,4, 1,2,5,8, 10,1

blockY
	byte 1,1,2,1, 4,2,3,10, 1,10

levelSize
	byte 64

; Pointer to each row in the screen pointer (x6 in this case)
RowPointer
	byte 0,8,16,24,32,40,48,56

; This points to each 10x10 cell in levelData
ScreenPointer
	byte <s11,<s12,<s13,<s14,<s15,<s16,<s17,<s18
	byte <s21,<s22,<s23,<s24,<s25,<s26,<s27,<s28
	byte <s31,<s32,<s33,<s34,<s35,<s36,<s37,<s38
	byte <s41,<s42,<s43,<s44,<s45,<s46,<s47,<s48
	byte <s51,<s52,<s53,<s54,<s55,<s56,<s57,<s58
	byte <s61,<s62,<s63,<s64,<s65,<s66,<s67,<s68
	byte <s71,<s72,<s73,<s74,<s75,<s76,<s77,<s78
	byte <s81,<s82,<s83,<s84,<s85,<s86,<s87,<s88
	byte <s90

tileType
	byte $11,$66,$CC,$DD
	byte $55,$44,$22,$EE

	org $FC00
LevelData
s11
	byte $F0
s12	
	byte $F0
s13
	byte $F0
s14
	byte $F0
s15
	byte $F0
s16
	byte $F0
s17
	byte $F0
s18
	byte $F0

s21
	byte $F0
s22
	byte $43,%00000111
	byte $32,%10110000
	byte $0b
	byte $b0
	byte $23,%00000000
	byte $22,%00011000
	byte $66,%10110000
s23
	byte $00,%00100000
	byte $D4 
	byte $52,%10110000
	byte $09,%00110000
s24
	byte $F0
s25
	byte $00,%00101000
	byte $02,%00100000
	byte $20,%00011000
	byte $C2,$D5
s26
	byte $02,%00100000
	byte $D0,$D5
s27
	byte $08,%00101000
	byte $04,%00100000
	byte $29,%00011000
	byte $D0
	byte $66,%10101000
	byte $44,%00000111
s28
	byte $F0

s31
	byte $F0
s32
	byte $a0
	byte $c2
	byte $d5
	byte $41, %00011000
s33
	byte $23,%00011010
	byte $33,%00000011
	byte $20,%00000001
	byte $35,%00000001
	byte $14,%00000001
	byte $50,%10110000
	byte $c6
s34
	byte $03,%00100000
	byte $07,%00101000
	byte $D0,$D5
s35
	byte $D0,$D5
s36
	byte $D0,$D5
s37
	byte $29,%00011000
	byte $D0,$D5
s38
	byte $F0

s41
	byte $F0
s42
	byte $40, %00011000
	byte $c2
	byte $d5
s43
	byte $0c
	byte $5c
s44
	byte $54,%00110000
	byte $63,%00011000
	byte $87,%00101000
	byte $91,%00010000
s45
	byte $0c
	byte $5c
s46
	byte $33,%10110000
s47
	byte $A9
	byte $58,%00011000
	byte $33,%10110000
s48
	byte $F0
s51
	byte $F0
s52
	byte $0c
	byte $5B
	byte $40,%00010000
	byte $70,%00010000
s53
	byte $4B
	byte $60,%00010000
	byte $26,%00000111
s54
	byte $F0
s55
	byte $30,%00011000
	byte $51,%00100010
	byte $64,%00100010
	byte $68,%00010010
s56
	byte $60,%00100010
	byte $54,%00000110
	byte $66,%00100010
	byte $53,%00000010
	byte $55,%00001010
s57
	byte $60,%00011010
	byte $33,%10110000
	byte $88,%00000111
s58
	byte $F0

s61
	byte $F0
s62
	byte $0D
	byte $50,%00110000
	byte $30,%00001000
	byte $83,%00001000
	byte $35,%10110000
s63
	byte $0D
	byte $54,%00110010
	byte $76,%00000011
s64
	byte $0D,$5D
	byte $01,%00100000
	byte $05,%00100000
	byte $14,%00010000
s65
	byte $0D
	byte $60,%00100001
	byte $90,%00100001
	byte $64,%00011001
	byte $71,%00101000
	byte $51,%00010001
	byte $83,%00000111
s66
	byte $0D,$5D
s67
	byte $A9
	byte $36, %00101000
	byte $28, %00011000
	byte $40, %00100010
	byte $00, %00011010
	byte $44, %00010010
s68
	byte $F0

s71
	byte $F0
s72
	byte $F0
s73
	byte $50,%00110000
	byte $30,%00001000
	byte $83,%00001000
	byte $25,%00011111
	byte $C7
s74
	byte $0D,$5D
	byte $9A
;	byte $83,%00100000
s75
	byte $0D,$5D
	byte $01,%00010001
;	byte $92,%00010000
s76
	byte $0D,$5D
	byte $9A
	byte $83,%00010000
s77
	byte $0D
	byte $57,%00110000
	byte $39,%00001000
	byte $86,%00001000
	byte $55,%00000111
s78
	byte $F0

s81
	byte $B0, $8B
	byte $02, %00010000
	byte $08, %00010000
	byte $68, %00110000
	byte $74, %00100000
	byte $52, %00000111
	byte $14, %10110000
s82
	byte $F0
s83
	byte $F0
s84
	byte $F0
s85
	byte $F0
s86
	byte $F0
s87
	byte $F0
s88
	byte $F0
s90


;---------------------------------------------------------------------------
; Program Startup
;---------------------------------------------------------------------------
	org $FFFC
	.word Start
	.word Start

